

import logging

from thanatos.database.db_utils import execute_sql

_log = logging.getLogger('thanatos')


def get_all_published_ships_basic(db_connection):
    """ Gets a list of all published ships and their basic information.

    :return: Each result has a tuple of (typeID, typeName, groupID, groupName, categoryID, and categoryName).
    :rtype: list
    """

    sql = 'CALL get_all_published_ships_basic();'
    results = execute_sql(sql, db_connection)

    return results


def get_dogma_attribute_for_type(db_connection, type_id, dogma_attribute):
    """

    :param type_id:
    :param dogma_attribute:

    :return:
    :rtype:
    """

    sql = 'CALL get_dogma_attribute_for_type({}, {});'.format(type_id, dogma_attribute)
    results = execute_sql(sql, db_connection, fetch='one')

    if results is not None:
        results = results[0]  # It's returned as a tuple from the DB, lets change that.

    return results





import mock
import unittest2

from thanatos.database import inventory


class DatabaseInventoryTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    @mock.patch('thanatos.database.inventory.execute_sql')
    def test_get_all_published_ships_basic(self, mock_execute_sql):
        """ Test we can call get all published ships basic. """

        mock_db_connection = mock.MagicMock()
        mock_execute_sql.return_value = [(1, 'test')]

        results = inventory.get_all_published_ships_basic(mock_db_connection)

        mock_execute_sql.assert_called_with('CALL get_all_published_ships_basic();', mock_db_connection)
        self.assertEqual(results, [(1, 'test')])

    @mock.patch('thanatos.database.inventory.execute_sql')
    def test_get_dogma_attribute_for_type(self, mock_execute_sql):
        """ Test we can call get specified dogma attribute for specified type ID. """

        mock_db_connection = mock.MagicMock()
        mock_type_id = 0
        mock_dogma_attribute = 111
        mock_execute_sql.return_value = (1, 'test')

        results = inventory.get_dogma_attribute_for_type(mock_db_connection, mock_type_id, mock_dogma_attribute)

        mock_execute_sql.assert_called_with('CALL get_dogma_attribute_for_type(0, 111);', mock_db_connection, fetch='one')
        self.assertEqual(results, 1)

    @mock.patch('thanatos.database.inventory.execute_sql')
    def test_get_dogma_attribute_for_type_handles_none(self, mock_execute_sql):
        """ Test we can call get specified dogma attribute for specified type ID. """

        mock_db_connection = mock.MagicMock()
        mock_type_id = 0
        mock_dogma_attribute = 111
        mock_execute_sql.return_value = None

        results = inventory.get_dogma_attribute_for_type(mock_db_connection, mock_type_id, mock_dogma_attribute)

        mock_execute_sql.assert_called_with('CALL get_dogma_attribute_for_type(0, 111);', mock_db_connection, fetch='one')
        self.assertEqual(results, None)




import mock
import MySQLdb
import unittest2

from thanatos.questions import universe


class BorderingRegionsTestCase(unittest2.TestCase):

    def setUp(self):
        self.mock_db_connection = mock.Mock(spec=MySQLdb.connection)

    def test_class_initializes(self):
        """ Simply test we can create an instance of the BRQ class. """

        universe.BorderingRegionsQuestion(self.mock_db_connection)

    @mock.patch('thanatos.questions.universe.universe')
    @mock.patch('thanatos.questions.base.Question.format_question')
    @mock.patch('random.choice')
    def test_question_ask(self, random_choice, mock_format_question, mock_db_universe):
        """ Test we can call the bordering region question ask method. """

        all_regions = [
            (10000001L, 'Region One'),
            (10000002L, 'Region Two'),
            (10000003L, 'Region Three'),
            (10000004L, 'Region Four'),
            (10000005L, 'Region Five'),
            (10000006L, 'Region Six'),
        ]

        mock_db_universe.get_all_not_wh_regions.return_value = all_regions

        random_choice.side_effect = [
            (10000001L, 'Region One'),
            (10000005L, 'Region Five'),
        ]

        mock_db_universe.get_all_regions_connected_to_region.return_value = [
            (10000005L, 'Region Five'),
            (10000006L, 'Region Six'),
        ]

        universe.BorderingRegionsQuestion(self.mock_db_connection).ask()

        random_choice.assert_any_call(all_regions)
        random_choice.assert_any_call([
            (10000005L, 'Region Five'),
            (10000006L, 'Region Six'),
        ])

        mock_format_question.assert_called_with(
            (10000005L, 'Region Five'),
            [(10000003L, 'Region Three'), (10000002L, 'Region Two'), (10000004L, 'Region Four')],
            'Which of the following regions borders the Region One region?',
        )


class PoitotTestCase(unittest2.TestCase):

    def setUp(self):
        self.mock_db_connection = mock.Mock(spec=MySQLdb.connection)

    def test_class_initializes(self):
        """ Simply test we can create an instance of the Poitot questions class. """

        universe.PoitotFamousForQuestion(self.mock_db_connection)

    @mock.patch('thanatos.questions.base.Question.format_question')
    def test_question_ask(self, mock_format_question):
        """ Test we can call the poitot question ask method. """

        universe.PoitotFamousForQuestion(self.mock_db_connection).ask()

        mock_format_question.assert_called_with(
            (0, 'The only named system in Syndicate.'),
            [(1, 'Kind to animals.'), (2, 'A fictional space detective.'), (3, 'Adjacent to F67E-Q.')],
            'Poitot is famous for being...?',
        )


class UniverseUtilsTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    def test_removal_of_regions(self):
        """ Specifically tests to make sure we remove the jove regions with no gates. """

        sample_regions_list = [
            (10000001L, 'Derelik'),
            (10000004L, 'UUA-F4'),
            (10000017L, 'J7HZ-F'),
            (10000019L, 'A821-A'),
        ]

        gateless_regions = universe.remove_regions_with_no_gates(sample_regions_list)

        self.assertEqual(gateless_regions, [(10000001L, 'Derelik')])

    def test_removal_of_regions_handles_region_not_in_list(self):
        """ Specifically tests we don't fail removing a region if it doesn't exist in the list. """

        sample_regions_list = [
            (10000001L, 'Derelik'),
            (10000004L, 'UUA-F4'),
        ]

        gateless_regions = universe.remove_regions_with_no_gates(sample_regions_list)

        self.assertEqual(gateless_regions, [(10000001L, 'Derelik')])




import random
import logging

from thanatos import categories
from thanatos.database import universe
from thanatos.questions.base import Question

_log = logging.getLogger('thanatos')


class BorderingRegionsQuestion(Question):
    """ Asks what region boards another given region. """

    name = 'Bordering Regions'
    description = 'Asks what region borders another region.'
    category = categories.geography
    sub_category = categories.geography_regions

    random_weight = 10

    question = 'Which of the following regions borders the {} region?'

    def ask(self):
        # Lets start by getting a region to base this all on and call it the source region
        # Lets ignore WH regions though
        all_regions = universe.get_all_not_wh_regions(self.db_connection)

        # Before we pick our source region we need to remove the Jove regions of as they have no gates:
        all_regions = remove_regions_with_no_gates(all_regions)

        # Now pick our random source region
        source_region = random.choice(all_regions)

        # Next lets find a random region that is connected to the source region, this will be the answer
        connected_regions = universe.get_all_regions_connected_to_region(self.db_connection, source_region[0])
        correct_answer = random.choice(connected_regions)

        # Now we need to find the possible wrong answers
        # These regions need to not be connected to the source region
        # And also not the source region itself
        regions_to_exclude = [x for x in connected_regions]
        regions_to_exclude.append(source_region)

        possible_wrong_answers = list(set(all_regions) - set(regions_to_exclude))

        question = self.format_question(correct_answer, possible_wrong_answers, self.question.format(source_region[1]))

        return question


class PoitotFamousForQuestion(Question):
    """ Asks what Poitot is famous for being. """

    name = 'Poitot'
    description = 'Asks what the Poitot solar system is famous for being.'
    category = categories.geography
    sub_category = categories.geography_miscellaneous

    random_weight = 1

    question = 'Poitot is famous for being...?'

    def ask(self):
        correct_answer = (0, 'The only named system in Syndicate.')

        # It would be great to add more here over time.
        possible_wrong_answers = [
            (1, 'Kind to animals.'),
            (2, 'A fictional space detective.'),
            (3, 'Adjacent to F67E-Q.'),
        ]

        question = self.format_question(correct_answer, possible_wrong_answers, self.question)

        return question


def remove_regions_with_no_gates(regions):
    """ Removes all Jove regions from a list of regions.

    :param regions: A list of tuples (regionID, regionName)
    :type regions: list

    :return: A list of regions minus those in jove space
    :rtype: list
    """

    list_of_gateless_regions = [
        (10000004, 'UUA-F4'),
        (10000017, 'J7HZ-F'),
        (10000019, 'A821-A'),
    ]

    for gateless_region in list_of_gateless_regions:
        if gateless_region in regions:
            regions.remove(gateless_region)

    return regions





import logging

from random import choice
from itertools import repeat

from thanatos.questions.base import Question

_log = logging.getLogger('thanatos')


def get_all_question_details():
    """

    :return: A dictionary of sets representing the categories and their sub categories.
    :rtype: dict
    """

    categories = {}
    subclasses = get_question_subclasses()
    
    for subclass in subclasses:
        if subclass.category is not None and subclass.sub_category is not None:
            name = subclass.__name__.lower()
            category = subclass.category['name'].lower()
            sub_category = subclass.sub_category['name'].lower()
            
            if category not in categories:
                categories[category] = subclass.category
                categories[category]['sub_categories'] = {}
            
            if sub_category not in categories[category]['sub_categories']:
                categories[category]['sub_categories'][sub_category] = subclass.sub_category
                categories[category]['sub_categories'][sub_category]['questions'] = {}
    
            categories[category]['sub_categories'][sub_category]['questions'][name] = {
                'name': subclass.name,
                'description': subclass.description,
                'weight': subclass.random_weight,
                'question': subclass.question,
            }
        
    return categories


def get_question_subclasses():
    return Question.__subclasses__()


def get_random_question():
    questions = []
    subclasses = get_question_subclasses()

    for subclass in subclasses:
        questions.extend(repeat(subclass, subclass.random_weight))

    return choice(questions)


def get_question(question_id):
    subclasses = get_question_subclasses()
    
    for subclass in subclasses:
        if subclass.__name__.lower() == question_id:
            return subclass
    
    raise InvalidQuestionException('The specified question does not exist.')


def get_question_from_category(category):
    questions = []
    subclasses = get_question_subclasses()

    for question in subclasses:
        if question.category is not None and question.sub_category is not None:
            if question.category['name'].lower() == category:
                questions.extend(repeat(question, question.random_weight))

                return choice(questions)

    raise InvalidQuestionCategoryException('The category specified is invalid.')


def get_question_from_sub_category(category, sub_category):
    questions = []
    subclasses = get_question_subclasses()

    for question in subclasses:
        if question.category is not None and question.sub_category is not None:
            if question.category['name'].lower() == category:
                if question.sub_category['name'].lower() == sub_category:
                    questions.extend(repeat(question, question.random_weight))

                    return choice(questions)

    raise InvalidQuestionCategoryException('Either the category or sub-category specified is invalid.')


class InvalidQuestionCategoryException(Exception):
    pass


class InvalidQuestionException(Exception):
    pass





import mock
import unittest2

from thanatos.questions import question_utils


class QuestionsUtilsTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    @mock.patch('thanatos.questions.question_utils.get_question_subclasses')
    @mock.patch('thanatos.questions.question_utils.choice')
    def test_get_random_question(self, mock_choice, mock_get_question_subclasses):
        """ Test to make sure the number in a list is multiplied by it's weight. """

        mock_question = mock.MagicMock()
        mock_question.random_weight = 3
        mock_get_question_subclasses.return_value = [mock_question]

        question_utils.get_random_question()

        mock_choice.assert_called_with([
            mock_question,
            mock_question,
            mock_question,
        ])

    def test_get_question(self):
        """ Test the get question method works. """

        question_utils.get_question('slotsquestion')

    def test_get_question_bad_question(self):
        """ Test the get question method returns a proper exception if an invalid question is specified. """

        self.failUnlessRaises(
            question_utils.InvalidQuestionException,
            question_utils.get_question,
            'invalidquestion',
        )

    def test_get_question_from_category(self):
        """ Test the get_question_from_category method works. """

        question_utils.get_question_from_category('inventory')

    def test_get_question_from_category_bad_category(self):
        """ Test the get_question_from_category method raises proper exception. """

        self.failUnlessRaises(
            question_utils.InvalidQuestionCategoryException,
            question_utils.get_question_from_category,
            'invalidcategory',
        )

    def test_get_question_from_sub_category(self):
        """ Test the get_question_from_sub_category method works. """

        question_utils.get_question_from_sub_category('inventory', 'slots')

    def test_get_question_from_sub_category_bad_category(self):
        """ Test the get_question_from_subcategory method raises the proper exception. """

        self.failUnlessRaises(
            question_utils.InvalidQuestionCategoryException,
            question_utils.get_question_from_sub_category,
            'invalidcategory',
            'invalidsubcategory',
        )

    def test_get_all_question_details(self):
        """ Test the get_all_question_details method works. """

        question_utils.get_all_question_details()




import os
import MySQLdb
import logging

_log = logging.getLogger('thanatos')


def execute_sql(sql, db_connection, fetch='all'):
    """

    :param sql:
    :param db_connection:
    :param fetch: A string of either 'all' or 'one'.

    :return:
    """

    cursor = db_connection.cursor()
    cursor.execute(sql)

    if fetch == 'all':
        results = [x for x in cursor.fetchall()]

    elif fetch == 'one':
        results = cursor.fetchone()

    else:
        results = None

    cursor.close()

    return results


def get_stored_procs(db_connection):
    """
    :param db_connection:
    :return:
    """

    sql = "SHOW PROCEDURE STATUS;"
    procs = execute_sql(sql, db_connection)

    return [x[1] for x in procs]

def drop_proc(proc_name, db_connection):
    """

    :param proc_name:
    :param db_connection:
    :return:
    """

    sql = "DROP PROCEDURE {};".format(proc_name)
    execute_sql(sql, db_connection)

def update_sql_stored_procs(db_connection):
    """

    :param db_connection:

    :return:
    """

    procs_dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'procs')
    existing_procs = get_stored_procs(db_connection)

    for proc_file_name in os.listdir(procs_dir_path):
        _log.info('Running {}'.format(proc_file_name))

        proc_file_path = os.path.join(procs_dir_path, proc_file_name)
        proc_name = proc_file_name.split('.')[1]

        if proc_name in existing_procs:
            drop_proc(proc_name, db_connection)

        with open(proc_file_path, 'r') as sql_file:
            sql = " ".join(sql_file.readlines())
            execute_sql(sql, db_connection, fetch=None)


def load_tables_from_files(db_connection):
    """ Looks in the current working directory for all required tables. """

    _log.info('Loading tables from disk to DB.')

    sde_dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'sde')

    for sde_file_name in os.listdir(sde_dir_path):
        _log.info('Loading the following table: {}'.format(sde_file_name))

        sde_file_path = os.path.join(sde_dir_path, sde_file_name)

        with open(sde_file_path, 'r') as sde_file:
            sql = sde_file.read()
            execute_sql(sql, db_connection)

    _log.info('Finished loading all requested tables.')


def get_connection(connection_details=None):
    """ Creates a connection to the MySQL DB. """

    if connection_details is None:
        connection_details = get_default_connection_details()

    return MySQLdb.connect(
        connection_details['host'],
        connection_details['user'],
        connection_details['password'],
        connection_details['database']
    )


def get_default_connection_details():
    """ Gets the connection details based on environment vars or Thanatos default settings.

    :return: Returns a dictionary of connection details.
    :rtype: dict
    """

    return {
        'host': os.environ.get('MYSQL_HOST', '127.0.0.1'),
        'user': os.environ.get('MYSQL_USER', 'vagrant'),
        'password': os.environ.get('MYSQL_PASSWORD', 'vagrant'),
        'database': os.environ.get('MYSQL_DB', 'thanatos'),
    }
